import sys

class Huffman:
  """
  Huffman encoding, shamelessly adapted by Luis A. Leiva from a script of Chris Meyers.
  See original code at http://openbookproject.net/py4fun/huffman/huffman.py.txt
  """
  
  def __init__(self):
    self.codes = {}
    
  def setup(self, data):
    tups = self.huffmanFreq(data)
    tree = self.huffmanTree(tups)    
    self._assignCodes(tree)
    self.tree  = tree
      
  def huffmanFreq(self, data):
    freqs  = self._frequency(data)
    tuples = self._sortFreq(freqs)
    return tuples
    
  def huffmanTree(self, tuples):
    tree = self._buildTree(tuples)
    tree = self._trimTree(tree)
    return tree
      
  def _frequency(self, data):
    freqs = {}
    for ch in data:
      freqs[ch] = freqs.get(ch,0) + 1
    return freqs

  def _sortFreq(self, freqs):
    tuples = [(freqs[c], c) for c in freqs.keys()]
    tuples.sort()
    return tuples

  def _buildTree(self, tuples):
    while len(tuples) > 1:
      leastTwo = tuple(tuples[0:2])
      theRest  = tuples[2:]
      combFreq = leastTwo[0][0] + leastTwo[1][0]  # Branch points freq
      tuples   = theRest + [(combFreq, leastTwo)] # Add branch point to the end
      tuples.sort()
    return tuples[0]

  def _trimTree(self, tree):
    p = tree[1]             # Ignore freq count in [0]
    if type(p) == type(""): # Leaf node, so return p
      return p
    else:
      return (self._trimTree(p[0]), self._trimTree(p[1]))
      
  def _assignCodes(self, node, pat=""):
    if type(node) == type(""):
      self.codes[node] = pat              # Leaf node: set its code
    else:
      self._assignCodes(node[0], pat+"0") # Branch point: do left branch
      self._assignCodes(node[1], pat+"1") # Then do the right branch
             
  def encode(self, data):
    self.setup(data)
    output = ""
    for ch in data: 
      output += self.codes[ch]
    return output

  def decode(self, data, tree=None):
    btree = self.tree if tree is None else tree
    output = ""
    pos = btree
    for bit in data:
      pos = pos[0] if bit == "0" else pos[1]
      if type(pos) == type(""):     
        output += pos
        pos = btree
    return output
