"""
Polling-based compression methods for cursor data, by Jeff Huang. 
Modified and documented by Luis A. Leiva.
Dual licensed under the MIT and GPL licenses.
"""

def compressByTimePolling(originalPoints, pollInterval=1000, smart=True):
  """
  originalPoints are [(timestamp,x,y), ...] tuples 
  pollInterval in ms (default: 1000)
  smart means to throw out points when the cursor hasn't moved (default: True)
  """
  firstPt     = originalPoints[0]
  compPoints  = [firstPt]
  currentTime = firstPt[0]
  for point in originalPoints:
    # Race condition in BPA logs.
    if point[0] >= currentTime + pollInterval:
      if smart and (compPoints[-1][1], compPoints[-1][2]) == (point[1], point[2]):
        continue
      currentTime += pollInterval
      compPoints.append((currentTime, point[1], point[2]))
  return compPoints

def compressByPausedPositions(originalPoints, pauseThreshold=250):
  """
  originalPoints are [(timestamp,x,y), ...] tuples
  pauseThreshold in ms (default: 250)
  """
  firstPt    = originalPoints[0]
  compPoints = [firstPt]
  prevPoint  = None
  for point in originalPoints:
    # Race condition in BPA logs.
    if prevPoint and point[0] >= prevPoint[0] + pauseThreshold:
      compPoints.append(prevPoint)
    prevPoint = point
  return compPoints
